# (c) Copyright 2018 Trent Hauck
# All Rights Reserved
"""Contains parser level packages."""

from gcgc.parser.base import SequenceParser
from gcgc.parser.base import EncodedSeqLengthParser

__all__ = ["SequenceParser", "EncodedSeqLengthParser"]
