# (c) Copyright 2018 Trent Hauck
# All Rights Reserved
"""The gcgc.encoded_seq module contains code related to the EncodedSeq object."""

from gcgc.encoded_seq.encoded_seq import EncodedSeq


__all__ = ["EncodedSeq"]
