# (c) Copyright 2018 Trent Hauck
# All Rights Reserved
"""Types of metadata to be associated with objects."""

from gcgc.fields.categorical_field import (
    LabelField,
    FileMetaDataField,
    AnnotationField,
    DescriptionField,
)

__all__ = ["LabelField", "FileMetaDataField", "AnnotationField", "DescriptionField"]
